#!/usr/bin/env python

# Capstone Python bindings, by Nguyen Anh Quynnh <aquynh@gmail.com>

from __future__ import print_function
from capstone import *
from capstone.riscv import *
from xprint import to_x, to_hex

RISCV_CODE32 = b"\x37\x34\x00\x00\x97\x82\x00\x00\xef\x00\x80\x00\xef\xf0\x1f\xff\xe7\x00\x45\x00\xe7\x00\xc0\xff\x63\x05\x41\x00\xe3\x9d\x61\xfe\x63\xca\x93\x00\x63\x53\xb5\x00\x63\x65\xd6\x00\x63\x76\xf7\x00\x03\x88\x18\x00\x03\x99\x49\x00\x03\xaa\x6a\x00\x03\xcb\x2b\x01\x03\xdc\x8c\x01\x23\x86\xad\x03\x23\x9a\xce\x03\x23\x8f\xef\x01\x93\x00\xe0\x00\x13\xa1\x01\x01\x13\xb2\x02\x7d\x13\xc3\x03\xdd\x13\xe4\xc4\x12\x13\xf5\x85\x0c\x13\x96\xe6\x01\x13\xd7\x97\x01\x13\xd8\xf8\x40\x33\x89\x49\x01\xb3\x0a\x7b\x41\x33\xac\xac\x01\xb3\x3d\xde\x01\x33\xd2\x62\x40\xb3\x43\x94\x00\x33\xe5\xc5\x00\xb3\x76\xf7\x00\xb3\x54\x39\x01\xb3\x50\x31\x00\x33\x9f\x0f\x00"
RISCV_CODE64 = b"\x13\x04\xa8\x7a"

all_tests = (
        (CS_ARCH_RISCV, CS_MODE_RISCV32, RISCV_CODE32, "riscv32"),
        (CS_ARCH_RISCV, CS_MODE_RISCV64, RISCV_CODE64, "riscv64"),
)


def print_insn_detail(insn):
    # print address, mnemonic and operands
    print("0x%x:\t%s\t%s" % (insn.address, insn.mnemonic, insn.op_str))

    # "data" instruction generated by SKIPDATA option has no detail
    if insn.id == 0:
        return

    if len(insn.operands) > 0:
        print("\top_count: %u" % len(insn.operands))
        c = 0
        for i in insn.operands:
            if i.type == RISCV_OP_REG:
                print("\t\toperands[%u].type: REG = %s" % (c, insn.reg_name(i.reg)))
            if i.type == RISCV_OP_IMM:
                print("\t\toperands[%u].type: IMM = 0x%s" % (c, to_x(i.imm)))
            if i.type == RISCV_OP_MEM:
                print("\t\toperands[%u].type: MEM" % c)
                if i.mem.base != 0:
                    print("\t\t\toperands[%u].mem.base: REG = %s" \
                        % (c, insn.reg_name(i.mem.base)))
                if i.mem.disp != 0:
                    print("\t\t\toperands[%u].mem.disp: 0x%s" \
                        % (c, to_x(i.mem.disp)))
            c += 1


# ## Test class Cs
def test_class():

    for (arch, mode, code, comment) in all_tests:
        print("*" * 16)
        print("Platform: %s" %comment)
        print("Code: %s" % to_hex(code))
        print("Disasm:")

        try:
            md = Cs(arch, mode)
            md.detail = True
            for insn in md.disasm(code, 0x1000):
                print_insn_detail(insn)
                print ()
            print("0x%x:\n" % (insn.address + insn.size))
        except CsError as e:
            print("ERROR: %s" %e)


if __name__ == '__main__':
    test_class()



#!/usr/bin/env python

# Capstone Python bindings, by Nguyen Anh Quynnh <aquynh@gmail.com>
from __future__ import print_function
from capstone import *
from capstone.x86 import *
from xprint import to_hex, to_x, to_x_32


X86_CODE64 = b"\x55\x48\x8b\x05\xb8\x13\x00\x00\xe9\xea\xbe\xad\xde\xff\x25\x23\x01\x00\x00\xe8\xdf\xbe\xad\xde\x74\xff"
X86_CODE16 = b"\x8d\x4c\x32\x08\x01\xd8\x81\xc6\x34\x12\x00\x00\x05\x23\x01\x00\x00\x36\x8b\x84\x91\x23\x01\x00\x00\x41\x8d\x84\x39\x89\x67\x00\x00\x8d\x87\x89\x67\x00\x00\xb4\xc6\x66\xe9\xb8\x00\x00\x00\x67\xff\xa0\x23\x01\x00\x00\x66\xe8\xcb\x00\x00\x00\x74\xfc"
X86_CODE32 = b"\x8d\x4c\x32\x08\x01\xd8\x81\xc6\x34\x12\x00\x00\x05\x23\x01\x00\x00\x36\x8b\x84\x91\x23\x01\x00\x00\x41\x8d\x84\x39\x89\x67\x00\x00\x8d\x87\x89\x67\x00\x00\xb4\xc6\xe9\xea\xbe\xad\xde\xff\xa0\x23\x01\x00\x00\xe8\xdf\xbe\xad\xde\x74\xff"

all_tests = (
        (CS_ARCH_X86, CS_MODE_16, X86_CODE16, "X86 16bit (Intel syntax)", None),
        (CS_ARCH_X86, CS_MODE_32, X86_CODE32, "X86 32 (AT&T syntax)", CS_OPT_SYNTAX_ATT),
        (CS_ARCH_X86, CS_MODE_32, X86_CODE32, "X86 32 (Intel syntax)", None),
        (CS_ARCH_X86, CS_MODE_64, X86_CODE64, "X86 64 (Intel syntax)", None),
        )


def get_eflag_name(eflag):
    if eflag == X86_EFLAGS_UNDEFINED_OF:
        return "UNDEF_OF"
    elif eflag == X86_EFLAGS_UNDEFINED_SF:
        return "UNDEF_SF"
    elif eflag == X86_EFLAGS_UNDEFINED_ZF:
        return "UNDEF_ZF"
    elif eflag == X86_EFLAGS_MODIFY_AF:
        return "MOD_AF"
    elif eflag == X86_EFLAGS_UNDEFINED_PF:
        return "UNDEF_PF"
    elif eflag == X86_EFLAGS_MODIFY_CF:
        return "MOD_CF"
    elif eflag == X86_EFLAGS_MODIFY_SF:
        return "MOD_SF"
    elif eflag == X86_EFLAGS_MODIFY_ZF:
        return "MOD_ZF"
    elif eflag == X86_EFLAGS_UNDEFINED_AF:
        return "UNDEF_AF"
    elif eflag == X86_EFLAGS_MODIFY_PF:
        return "MOD_PF"
    elif eflag == X86_EFLAGS_UNDEFINED_CF:
        return "UNDEF_CF"
    elif eflag == X86_EFLAGS_MODIFY_OF:
        return "MOD_OF"
    elif eflag == X86_EFLAGS_RESET_OF:
        return "RESET_OF"
    elif eflag == X86_EFLAGS_RESET_CF:
        return "RESET_CF"
    elif eflag == X86_EFLAGS_RESET_DF:
        return "RESET_DF"
    elif eflag == X86_EFLAGS_RESET_IF:
        return "RESET_IF"
    elif eflag == X86_EFLAGS_TEST_OF:
        return "TEST_OF"
    elif eflag == X86_EFLAGS_TEST_SF:
        return "TEST_SF"
    elif eflag == X86_EFLAGS_TEST_ZF:
        return "TEST_ZF"
    elif eflag == X86_EFLAGS_TEST_PF:
        return "TEST_PF"
    elif eflag == X86_EFLAGS_TEST_CF:
        return "TEST_CF"
    elif eflag == X86_EFLAGS_RESET_SF:
        return "RESET_SF"
    elif eflag == X86_EFLAGS_RESET_AF:
        return "RESET_AF"
    elif eflag == X86_EFLAGS_RESET_TF:
        return "RESET_TF"
    elif eflag == X86_EFLAGS_RESET_NT:
        return "RESET_NT"
    elif eflag == X86_EFLAGS_PRIOR_OF:
        return "PRIOR_OF"
    elif eflag == X86_EFLAGS_PRIOR_SF:
        return "PRIOR_SF"
    elif eflag == X86_EFLAGS_PRIOR_ZF:
        return "PRIOR_ZF"
    elif eflag == X86_EFLAGS_PRIOR_AF:
        return "PRIOR_AF"
    elif eflag == X86_EFLAGS_PRIOR_PF:
        return "PRIOR_PF"
    elif eflag == X86_EFLAGS_PRIOR_CF:
        return "PRIOR_CF"
    elif eflag == X86_EFLAGS_PRIOR_TF:
        return "PRIOR_TF"
    elif eflag == X86_EFLAGS_PRIOR_IF:
        return "PRIOR_IF"
    elif eflag == X86_EFLAGS_PRIOR_DF:
        return "PRIOR_DF"
    elif eflag == X86_EFLAGS_TEST_NT:
        return "TEST_NT"
    elif eflag == X86_EFLAGS_TEST_DF:
        return "TEST_DF"
    elif eflag == X86_EFLAGS_RESET_PF:
        return "RESET_PF"
    elif eflag == X86_EFLAGS_PRIOR_NT:
        return "PRIOR_NT"
    elif eflag == X86_EFLAGS_MODIFY_TF:
        return "MOD_TF"
    elif eflag == X86_EFLAGS_MODIFY_IF:
        return "MOD_IF"
    elif eflag == X86_EFLAGS_MODIFY_DF:
        return "MOD_DF"
    elif eflag == X86_EFLAGS_MODIFY_NT:
        return "MOD_NT"
    elif eflag == X86_EFLAGS_MODIFY_RF:
        return "MOD_RF"
    elif eflag == X86_EFLAGS_SET_CF:
        return "SET_CF"
    elif eflag == X86_EFLAGS_SET_DF:
        return "SET_DF"
    elif eflag == X86_EFLAGS_SET_IF:
        return "SET_IF"
    else: 
        return None


def print_insn_detail(mode, insn):
    def print_string_hex(comment, str):
        print(comment, end=' '),
        for c in str:
            print("0x%02x " % c, end=''),
        print()

    # print address, mnemonic and operands
    print("0x%x:\t%s\t%s" % (insn.address, insn.mnemonic, insn.op_str))

    # "data" instruction generated by SKIPDATA option has no detail
    if insn.id == 0:
        return

    # print instruction prefix
    print_string_hex("\tPrefix:", insn.prefix)

    # print instruction's opcode
    print_string_hex("\tOpcode:", insn.opcode)

    # print operand's REX prefix (non-zero value is relavant for x86_64 instructions)
    print("\trex: 0x%x" % (insn.rex))

    # print operand's address size
    print("\taddr_size: %u" % (insn.addr_size))

    # print modRM byte
    print("\tmodrm: 0x%x" % (insn.modrm))

    # print modRM offset
    if insn.encoding.modrm_offset != 0:
        print("\tmodrm_offset: 0x%x" % (insn.encoding.modrm_offset))

    # print displacement value
    print("\tdisp: 0x%s" % to_x_32(insn.disp))

    # print displacement offset (offset into instruction bytes)
    if insn.encoding.disp_offset != 0:
        print("\tdisp_offset: 0x%x" % (insn.encoding.disp_offset))

    # print displacement size
    if insn.encoding.disp_size != 0:
        print("\tdisp_size: 0x%x" % (insn.encoding.disp_size))

    # SIB is not available in 16-bit mode
    if (mode & CS_MODE_16 == 0):
        # print SIB byte
        print("\tsib: 0x%x" % (insn.sib))
        if (insn.sib):
            if insn.sib_base != 0:
                print("\t\tsib_base: %s" % (insn.reg_name(insn.sib_base)))
            if insn.sib_index != 0:
                print("\t\tsib_index: %s" % (insn.reg_name(insn.sib_index)))
            if insn.sib_scale != 0:
                print("\t\tsib_scale: %d" % (insn.sib_scale))

    # XOP CC type
    if insn.xop_cc != X86_XOP_CC_INVALID:
        print("\txop_cc: %u" % (insn.xop_cc))

    # SSE CC type
    if insn.sse_cc != X86_SSE_CC_INVALID:
        print("\tsse_cc: %u" % (insn.sse_cc))

    # AVX CC type
    if insn.avx_cc != X86_AVX_CC_INVALID:
        print("\tavx_cc: %u" % (insn.avx_cc))

    # AVX Suppress All Exception
    if insn.avx_sae:
        print("\tavx_sae: TRUE")

    # AVX Rounding Mode type
    if insn.avx_rm != X86_AVX_RM_INVALID:
        print("\tavx_rm: %u" % (insn.avx_rm))

    count = insn.op_count(X86_OP_IMM)
    if count > 0:
        print("\timm_count: %u" % count)
        for i in range(count):
            op = insn.op_find(X86_OP_IMM, i + 1)
            print("\t\timms[%u]: 0x%s" % (i + 1, to_x(op.imm)))
            if insn.encoding.imm_offset != 0:
                print("\timm_offset: 0x%x" % (insn.encoding.imm_offset))
            if insn.encoding.imm_size != 0:
                print("\timm_size: 0x%x" % (insn.encoding.imm_size))

    if len(insn.operands) > 0:
        print("\top_count: %u" % len(insn.operands))
        c = -1
        for i in insn.operands:
            c += 1
            if i.type == X86_OP_REG:
                print("\t\toperands[%u].type: REG = %s" % (c, insn.reg_name(i.reg)))
            if i.type == X86_OP_IMM:
                print("\t\toperands[%u].type: IMM = 0x%s" % (c, to_x(i.imm)))
            if i.type == X86_OP_MEM:
                print("\t\toperands[%u].type: MEM" % c)
                if i.mem.segment != 0:
                    print("\t\t\toperands[%u].mem.segment: REG = %s" % (c, insn.reg_name(i.mem.segment)))
                if i.mem.base != 0:
                    print("\t\t\toperands[%u].mem.base: REG = %s" % (c, insn.reg_name(i.mem.base)))
                if i.mem.index != 0:
                    print("\t\t\toperands[%u].mem.index: REG = %s" % (c, insn.reg_name(i.mem.index)))
                if i.mem.scale != 1:
                    print("\t\t\toperands[%u].mem.scale: %u" % (c, i.mem.scale))
                if i.mem.disp != 0:
                    print("\t\t\toperands[%u].mem.disp: 0x%s" % (c, to_x(i.mem.disp)))

            # AVX broadcast type
            if i.avx_bcast != X86_AVX_BCAST_INVALID:
                print("\t\toperands[%u].avx_bcast: %u" % (c, i.avx_bcast))

            # AVX zero opmask {z}
            if i.avx_zero_opmask:
                print("\t\toperands[%u].avx_zero_opmask: TRUE" % (c))

            print("\t\toperands[%u].size: %u" % (c, i.size))

            if i.access == CS_AC_READ:
                print("\t\toperands[%u].access: READ\n" % (c))
            elif i.access == CS_AC_WRITE:
                print("\t\toperands[%u].access: WRITE\n" % (c))
            elif i.access == CS_AC_READ | CS_AC_WRITE:
                print("\t\toperands[%u].access: READ | WRITE\n" % (c))

    (regs_read, regs_write) = insn.regs_access()

    if len(regs_read) > 0:
        print("\tRegisters read:", end="")
        for r in regs_read:
            print(" %s" %(insn.reg_name(r)), end="")
        print("")

    if len(regs_write) > 0:
        print("\tRegisters modified:", end="")
        for r in regs_write:
            print(" %s" %(insn.reg_name(r)), end="")
        print("")
        
    if insn.eflags:
        updated_flags = []
        for i in range(0,46):
            if insn.eflags & (1 << i):
                updated_flags.append(get_eflag_name(1 << i))
        print("\tEFLAGS: %s" % (','.join(p for p in updated_flags)))
        

# ## Test class Cs
def test_class():

    for (arch, mode, code, comment, syntax) in all_tests:
        print("*" * 16)
        print("Platform: %s" % comment)
        print("Code: %s" % to_hex(code))
        print("Disasm:")

        try:
            md = Cs(arch, mode)
            md.detail = True

            if syntax is not None:
                md.syntax = syntax

            for insn in md.disasm(code, 0x1000):
                print_insn_detail(mode, insn)
                print ()
            print ("0x%x:\n" % (insn.address + insn.size))
        except CsError as e:
            print("ERROR: %s" % e)


if __name__ == '__main__':
    test_class()



#!/usr/bin/env python3

import sys
import re
import argparse
from pathlib import Path

# 80001c1a <IfxVadc_disableAccess>:
# 80001c1a:	40 4f       	mov.aa %a15,%a4
# 80001c1c:	02 48       	mov %d8,%d4
# 80001c1e:	6d ff 9d ff 	call 80001b58 <IfxScuWdt_getSafetyWatchdogPassword>

unique_set = set()


def num2prefix_hex(x, prefix="#"):
    if x.startswith("0x") or x.startswith("-0x") or x == "0":
        x = prefix + x
    if x.isdigit() or (x.startswith("-") and x[1:].isdigit()):
        x = prefix + hex(int(x))
    return x

def op2prefix_hex(x):
    x = num2prefix_hex(x)
    if "]" in x:
        xs = x.split("]")
        if xs[1].isdigit() or xs[1].startswith('-'):
            x = xs[0] + "]" + num2prefix_hex(xs[1])
    return x

def gen(filename):
    with open(filename, "r") as f:
        for line in f:
            caps = re.findall(
                r"([0-9a-f]+):\s+([0-9a-f]+) ([0-9a-f]+) ([0-9a-f]+)? ([0-9a-f]+)?\s+"
                r"(\S+) (\S+)",
                line,
            )
            if not caps:
                continue
            caps = caps[0]
            addr = int(caps[0], 16)
            hexstr = caps[1:5]
            mnemonic: str = caps[5]
            operands = caps[6]

            def try_dedisp(x):
                try:
                    disp = int(x, 16)
                    if disp > 0x80000000:
                        x = hex(disp - addr)
                    return x
                except ValueError:
                    pass
                return x

            def is_hex_string(s: str) -> bool:
                if not s.isalnum():
                    return False
                return all(c.isdigit() or c.lower() in "abcdef" for c in s) and any(
                    c.lower() in "abcdef" for c in s
                )

            hexstr = ",".join(f"0x{x}" for x in hexstr if x)
            fun = re.match(r"\s*<.+>\s*", operands)
            # print(hex(addr), hexstr, mnemonic, operands)
            if any(
                [
                    mnemonic.startswith(pre)
                    for pre in [
                        "mtcr",
                        "mfcr",
                        "st.a",
                        "st.b",
                        "st.d",
                        "st.w",
                        "ld.a",
                        "ld.b",
                        "ld.d",
                        "ld.w",
                    ]
                ]
            ):
                # unique_set.add(f"# {hexstr.ljust(19)} = {mnemonic}\t{operands}")
                continue

            ops = operands.split(",")
            if (
                any(
                    [mnemonic.startswith(pre) for pre in ["j", "call", "loop", "fcall"]]
                )
                or fun
            ):
                re.sub(r"\s*<.+>\s*", "", operands)
                # de relative addressing
                ops = list(map(try_dedisp, ops))

            for i, x in enumerate(ops):
                if is_hex_string(x) and not x.startswith("0x"):
                    x = "#0x" + x
                x = op2prefix_hex(x)
                ops[i] = x

            operands = ", ".join(ops)
            operands = operands.replace("%", "")
            unique_set.add(f"{hexstr.ljust(19)} = {mnemonic}\t{operands}")

    print("# CS_ARCH_TRICORE, CS_MODE_TRICORE_162, None")
    print("\n".join(unique_set))

def att2intel(filename):
    with open(filename, "r") as fp:
        lines = []
        for line in fp.readlines():
            if not '=' in line:
                lines.append(line)
                continue
            insn = line.split('=')
            hexstr = insn[0]
            insn = insn[1]
            ops = insn.strip().split(', ')
            ops = ops[0].split('\t') + ops[1:]
            mnemonic = ops[0]
            ops = ops[1:]
            for i,op in enumerate(ops):
                op = op.strip()
                op = op2prefix_hex(op)
                ops[i] = op
            operands = ", ".join(ops)
            lines.append(f"{hexstr.ljust(19)} = {mnemonic}\t{operands}")
        print('\n'.join(lines))

def main():
    parser = argparse.ArgumentParser(description="Convert objdump's output to .s.cs test file")
    parser.add_argument('input', type=Path, help='input file path')
    parser.add_argument('--intel', action='store_true', help='convert .s.cs file to intel syntax')
    args = parser.parse_args()
    if not args.intel:
        gen(args.input)
    else:
        att2intel(args.input)


if __name__ == "__main__":
    main()



import unittest
from capstone import *
from capstone.arm64 import *
from collections import defaultdict

class ARM64PACRegAccessTest(unittest.TestCase):

    PATTERNS = [
        ("41 00 C1 DA", "pacia x1, x2"),
        ("3F 23 03 D5", "paciasp"),
        ("E1 23 C1 DA", "paciza x1"),
        ("41 04 C1 DA", "pacib x1, x2"),
        ("7F 23 03 D5", "pacibsp"),
        ("E1 27 C1 DA", "pacizb x1"),
        ("41 08 C1 DA", "pacda x1, x2"),
        ("E1 2B C1 DA", "pacdza x1"),
        ("41 0C C1 DA", "pacdb x1, x2"),
        ("E1 2F C1 DA", "pacdzb x1"),
        ("41 18 C1 DA", "autda x1, x2"),
        ("E1 3B C1 DA", "autdza x1"),
        ("41 1C C1 DA", "autdb x1, x2"),
        ("E1 3F C1 DA", "autdzb x1"),
        ("41 10 C1 DA", "autia x1, x2"),
        ("BF 23 03 D5", "autiasp"),
        ("E1 33 C1 DA", "autiza x1"),
        ("9F 23 03 D5", "autiaz"),
        ("41 14 C1 DA", "autib x1, x2"),
        ("FF 23 03 D5", "autibsp"),
        ("E1 37 C1 DA", "autizb x1"),
        ("DF 23 03 D5", "autibz"),
        ("E1 47 C1 DA", "xpacd x1"),
        ("E1 43 C1 DA", "xpaci x1"),
        ("FF 20 03 D5", "xpaclri"),
        ]

    def setUp(self):
        self.insts = []
        self.cs = Cs(CS_ARCH_ARM64, CS_MODE_LITTLE_ENDIAN)
        self.cs.detail = True

        for pattern, asm in self.PATTERNS:
            # Disassemble the instruction. Any error here means Capstone doesn't handle the instruction (maybe the wrong branch)
            inst = next(self.cs.disasm(bytes.fromhex(pattern), 0))

            # Build the lists of expected read and written registers
            regs = list(map(lambda r: r.strip(', '), asm.split()[1:]))
            expected_regs_read = []
            n = len(regs)
            if(n == 0):
                expected_regs_written = ["lr"]
                expected_regs_read = ["lr"]
                if(asm.endswith("sp")):
                    expected_regs_read += ["sp"]
            elif(n == 1):
                expected_regs_written = [regs[0]]
                expected_regs_read = [regs[0]]
            elif(n == 2):
                expected_regs_written = [regs[0]]
                expected_regs_read = regs
            
            expected_regs = [expected_regs_read, expected_regs_written]
            #print((inst, asm, expected_regs))

            self.insts.append((inst, asm, expected_regs))


    def test_regs_access(self):
        """Check that the `regs_access` API provides correct data"""
        
        for inst, asm, expected_regs in self.insts:
            for i, decoded_regs in enumerate(map(lambda l: list(map(self.cs.reg_name, l)), inst.regs_access())):
                self.assertEqual(set(decoded_regs), set(expected_regs[i]), "%s has %r %s registers instead of %r" % (asm, decoded_regs, ["read", "written"][i], expected_regs[i]))


    def test_operands(self):
        """Check that the `operands` API provides correct data"""
        for inst, asm, expected_regs in self.insts:
            ops = inst.operands
            asm_regs = list(map(lambda r: r.strip(', '), asm.split()[1:]))
            self.assertEqual(len(ops), len(asm_regs))

            expected_regs_accesses = defaultdict(int)
            
            expected_regs_read, expected_regs_written = expected_regs
            for reg in expected_regs_written:
                expected_regs_accesses[reg] |= CS_AC_WRITE
            for reg in expected_regs_read:
                expected_regs_accesses[reg] |= CS_AC_READ
            
            for i, op in enumerate(ops):
                self.assertEqual(op.type, CS_OP_REG, "%s has operand %d with invalid type" % (asm, i))
                regname = self.cs.reg_name(op.reg)
                self.assertEqual(op.access, expected_regs_accesses[regname], "%s has operand %d (%s) with invalid access (%d != %d)" % (asm, i, regname, op.access, expected_regs_accesses[regname]))

            
if __name__ == '__main__':
    unittest.main()



# Capstone Python bindings, by billow <billow.fun@gmail.com>

import ctypes
from . import copy_ctypes_list
from .tricore_const import *

class TriCoreOpMem(ctypes.Structure):
    _fields_ = (
        ('base', ctypes.c_uint8),
        ('disp', ctypes.c_int32),
    )


class TriCoreOpValue(ctypes.Union):
    _fields_ = (
        ('reg', ctypes.c_uint),
        ('imm', ctypes.c_int32),
        ('mem', TriCoreOpMem),
    )


class TriCoreOp(ctypes.Structure):
    _fields_ = (
        ('type', ctypes.c_uint),
        ('value', TriCoreOpValue),
        ('access', ctypes.c_uint8)
    )

    @property
    def imm(self):
        return self.value.imm

    @property
    def reg(self):
        return self.value.reg

    @property
    def mem(self):
        return self.value.mem


# Instruction structure
class CsTriCore(ctypes.Structure):
    _fields_ = (
        ('op_count', ctypes.c_uint8),
        ('operands', TriCoreOp * 8),
        ('update_flags', ctypes.c_bool),
    )

def get_arch_info(a):
    return (a.update_flags, copy_ctypes_list(a.operands[:a.op_count]))



#!/usr/bin/python
# convert LLVM GenAsmWriter.inc for Capstone disassembler.
# by Nguyen Anh Quynh, 2019

import sys

if len(sys.argv) == 1:
    print("Syntax: %s <GenAsmWriter.inc> <Output-GenAsmWriter.inc> <Output-GenRegisterName.inc> <arch>" %sys.argv[0])
    sys.exit(1)

arch = sys.argv[4] 
f = open(sys.argv[1])
lines = f.readlines()
f.close()

f1 = open(sys.argv[2], 'w+')

f2 = open(sys.argv[3], 'w+')

f1.write("/* Capstone Disassembly Engine, http://www.capstone-engine.org */\n")
f1.write("/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2019 */\n")
f1.write("\n")

f2.write("/* Capstone Disassembly Engine, http://www.capstone-engine.org */\n")
f2.write("/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2019 */\n")
f2.write("\n")

need_endif = False
in_getRegisterName = False
in_printAliasInstr = False
fragment_no = None
skip_printing = False

skip_line = 0
skip_count = 0

def replace_getOp(line):
    line2 = line
    if 'MI->getOperand(0)' in line:
        line2 = line.replace('MI->getOperand(0)', 'MCInst_getOperand(MI, 0)')
    elif 'MI->getOperand(1)' in line:
        line2 = line.replace('MI->getOperand(1)', 'MCInst_getOperand(MI, 1)')
    elif 'MI->getOperand(2)' in line:
        line2 = line.replace('MI->getOperand(2)', 'MCInst_getOperand(MI, 2)')
    elif 'MI->getOperand(3)' in line:
        line2 = line.replace('MI->getOperand(3)', 'MCInst_getOperand(MI, 3)')
    elif 'MI->getOperand(4)' in line:
        line2 = line.replace('MI->getOperand(4)', 'MCInst_getOperand(MI, 4)')
    elif 'MI->getOperand(5)' in line:
        line2 = line.replace('MI->getOperand(5)', 'MCInst_getOperand(MI, 5)')
    elif 'MI->getOperand(6)' in line:
        line2 = line.replace('MI->getOperand(6)', 'MCInst_getOperand(MI, 6)')
    elif 'MI->getOperand(7)' in line:
        line2 = line.replace('MI->getOperand(7)', 'MCInst_getOperand(MI, 7)')
    elif 'MI->getOperand(8)' in line:
        line2 = line.replace('MI->getOperand(8)', 'MCInst_getOperand(MI, 8)')
    return line2

def replace_getReg(line):
    line2 = line
    if 'MI->getOperand(0).getReg()' in line:
        line2 = line.replace('MI->getOperand(0).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 0))')
    elif 'MI->getOperand(1).getReg()' in line:
        line2 = line.replace('MI->getOperand(1).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 1))')
    elif 'MI->getOperand(2).getReg()' in line:
        line2 = line.replace('MI->getOperand(2).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 2))')
    elif 'MI->getOperand(3).getReg()' in line:
        line2 = line.replace('MI->getOperand(3).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 3))')
    elif 'MI->getOperand(4).getReg()' in line:
        line2 = line.replace('MI->getOperand(4).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 4))')
    elif 'MI->getOperand(5).getReg()' in line:
        line2 = line.replace('MI->getOperand(5).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 5))')
    elif 'MI->getOperand(6).getReg()' in line:
        line2 = line.replace('MI->getOperand(6).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 6))')
    elif 'MI->getOperand(7).getReg()' in line:
        line2 = line.replace('MI->getOperand(7).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 7))')
    elif 'MI->getOperand(8).getReg()' in line:
        line2 = line.replace('MI->getOperand(8).getReg()', 'MCOperand_getReg(MCInst_getOperand(MI, 8))')
    return line2

# extract param between text()
# MRI.getRegClass(AArch64::GPR32spRegClassID).contains(MI->getOperand(1).getReg()))
def extract_paren(line, text):
    i = line.index(text)
    return line[line.index('(', i)+1 : line.index(')', i)]


# extract text between <>
# printSVERegOp<'q'>
def extract_brackets(line):
    if '<' in line:
        return line[line.index('<')+1 : line.index('>')]
    else:
        return ''

# delete text between <>, including <>
# printSVERegOp<'q'>
def del_brackets(line):
    if '<' in line:
        return line[:line.index('<')] + line[line.index('>') + 1:]
    else:
        return line


def print_line(line):
    line = line.replace('::', '_')
    line = line.replace('nullptr', 'NULL')
    if not skip_printing:
        if in_getRegisterName:
            f2.write(line + "\n")
        else:
            f1.write(line + "\n")


for line in lines:
    line = line.rstrip()
    #print("@", line)

    # skip Alias
    if arch.upper() == 'X86':
        if 'PRINT_ALIAS_INSTR' in line:
            # done
            break

    if skip_line:
        skip_count += 1
        if skip_count <= skip_line:
            # skip this line
            continue
        else:
            # skip enough number of lines, reset counters
            skip_line = 0
            skip_count = 0

    if "::printInstruction" in line:
        if arch.upper() in ('AARCH64', 'ARM64'):
            #print_line("static void printInstruction(MCInst *MI, SStream *O, MCRegisterInfo *MRI)\n{")
            print_line("static void printInstruction(MCInst *MI, SStream *O)\n{")
        else:
            print_line("static void printInstruction(MCInst *MI, SStream *O)\n{")
    elif 'LLVM_NO_PROFILE_INSTRUMENT_FUNCTION' in line:
        continue
    elif 'AArch64InstPrinter::getMnemonic' in line:
        print_line("static uint64_t getMnemonic(MCInst *MI, SStream *O, unsigned int opcode) {")
    elif 'return {AsmStrs+(Bits' in line:
        tmp = line.split(',')
        prntStr = tmp[0].split('{')[1]
        print_line("\tSStream_concat0(O, " + prntStr + ");")
        print_line("\treturn Bits;")
    elif 'MnemonicInfo = getMnemonic(' in line:
        continue
    elif 'O << MnemonicInfo' in line:
        continue
    elif 'uint64_t Bits = MnemonicInfo' in line:
        print_line("\tuint64_t Bits = getMnemonic(MI, O, opcode);")
    elif 'const char *AArch64InstPrinter::' in line:
        continue
    elif 'getRegisterName(' in line:
        if 'unsigned AltIdx' in line:
            print_line("static const char *getRegisterName(unsigned RegNo, unsigned AltIdx)\n{")
        else:
            print_line("static const char *getRegisterName(unsigned RegNo)\n{")
    elif 'getRegisterName' in line:
        in_getRegisterName = True
        print_line(line)
    elif '::printAliasInstr' in line:
        if arch.upper() in ('AARCH64', 'PPC'):
            print_line("static char *printAliasInstr(MCInst *MI, SStream *OS, MCRegisterInfo *MRI)\n{")
            print_line('  #define GETREGCLASS_CONTAIN(_class, _reg) MCRegisterClass_contains(MCRegisterInfo_getRegClass(MRI, _class), MCOperand_getReg(MCInst_getOperand(MI, _reg)))')
        else:
            print_line("static bool printAliasInstr(MCInst *MI, SStream *OS)\n{")
        print_line("  unsigned int I = 0, OpIdx, PrintMethodIdx;")
        print_line("  char *tmpString;")
        in_printAliasInstr = True
    elif 'STI.getFeatureBits()[' in line:
        if arch.upper() == 'ARM':
            line2 = line.replace('STI.getFeatureBits()[', 'ARM_getFeatureBits(MI->csh->mode, ')
        elif arch.upper() == 'AARCH64':
            line2 = line.replace('STI.getFeatureBits()[', 'AArch64_getFeatureBits(')
        line2 = line2.replace(']', ')')
        print_line(line2)
    elif 'lookupBTIByEncoding' in line:
        line = line.replace('AArch64BTIHint::', '')
        line = line.replace('MCOp.getImm()', 'MCOperand_getImm(MCOp)')
        print_line(line)
    elif 'lookupPSBByEncoding' in line:
        line = line.replace('AArch64PSBHint::', '')
        line = line.replace('MCOp.getImm()', 'MCOperand_getImm(MCOp)')
        print_line(line)
    elif ', STI, ' in line:
        line2 = line.replace(', STI, ', ', ')

        if 'printSVELogicalImm<' in line:
            if 'int16' in line:
                line2 = line2.replace('printSVELogicalImm', 'printSVELogicalImm16')
                line2 = line2.replace('<int16_t>', '')
            elif 'int32' in line: 
                line2 = line2.replace('printSVELogicalImm', 'printSVELogicalImm32')
                line2 = line2.replace('<int32_t>', '')
            else:
                line2 = line2.replace('printSVELogicalImm', 'printSVELogicalImm64')
                line2 = line2.replace('<int64_t>', '')

        if 'MI->getOperand(' in line:
            line2 = replace_getOp(line2)

        # C++ template
        if 'printPrefetchOp' in line2:
            param = extract_brackets(line2)
            if param == '':
                param = 'false'
            line2 = del_brackets(line2)
            line2 = line2.replace(', O);', ', O, %s);' %param)
            line2 = line2.replace(', OS);', ', OS, %s);' %param)
        elif '<false>' in line2:
            line2 = line2.replace('<false>', '')
            line2 = line2.replace(', O);', ', O, false);')
            line2 = line2.replace('STI, ', '')
        elif '<true>' in line:
            line2 = line2.replace('<true>', '')
            line2 = line2.replace(', O);', ', O, true);')
            line2 = line2.replace('STI, ', '')
        elif 'printAdrLabelOperand' in line:
            # C++ template
            if '<0>' in line:
                line2 = line2.replace('<0>', '')
                line2 = line2.replace(', O);', ', O, 0);')
            elif '<1>' in line:
                line2 = line2.replace('<1>', '')
                line2 = line2.replace(', O);', ', O, 1);')
            elif '<2>' in line:
                line2 = line2.replace('<2>', '')
                line2 = line2.replace(', O);', ', O, 2);')
        elif 'printImm8OptLsl' in line2:
            param = extract_brackets(line2)
            line2 = del_brackets(line2)
            if '8' in param or '16' in param or '32' in param:
                line2 = line2.replace('printImm8OptLsl', 'printImm8OptLsl32')
            elif '64' in param:
                line2 = line2.replace('printImm8OptLsl', 'printImm8OptLsl64')
        elif 'printLogicalImm' in line2:
            param = extract_brackets(line2)
            line2 = del_brackets(line2)
            if '8' in param or '16' in param or '32' in param:
                line2 = line2.replace('printLogicalImm', 'printLogicalImm32')
            elif '64' in param:
                line2 = line2.replace('printLogicalImm', 'printLogicalImm64')
        elif 'printSVERegOp' in line2 or 'printGPRSeqPairsClassOperand' in line2 or 'printTypedVectorList' in line2 or 'printPostIncOperand' in line2 or 'printImmScale' in line2 or 'printRegWithShiftExtend' in line2 or 'printUImm12Offset' in line2 or 'printExactFPImm' in line2 or 'printMemExtend' in line2 or 'printZPRasFPR' in line2 or 'printMatrixTileVector' in line2 or 'printMatrix<' in line2 or 'printSImm' in line2:
            param = extract_brackets(line2)
            if param == '':
                param = '0'
            line2 = del_brackets(line2)
            line2 = line2.replace(', O);', ', O, %s);' %param)
            line2 = line2.replace(', OS);', ', OS, %s);' %param)
        elif 'printComplexRotationOp' in line:
            # printComplexRotationOp<90, 0>(MI, 5, STI, O);
            bracket_content = line2[line2.index('<') + 1 : line2.index('>')]
            line2 = line2.replace('<' + bracket_content + '>', '')
            line2 = line2.replace(' O);', ' O, %s);' %bracket_content)
        elif 'printAlignedLabel' in line2 or 'printAdrpLabel' in line2:
            line2 = line2.replace('Address, ', '')

        print_line(line2)
    elif "static const char AsmStrs[]" in line:
        print_line("#ifndef CAPSTONE_DIET")
        print_line("  static const char AsmStrs[] = {")
        need_endif = True
    elif "static const char AsmStrsNoRegAltName[]" in line:
        print_line("#ifndef CAPSTONE_DIET")
        print_line("  static const char AsmStrsNoRegAltName[] = {")
        need_endif = True
    elif line == '  O << "\\t";':
        print_line("  unsigned int opcode = MCInst_getOpcode(MI);")
        print_line('  // printf("opcode = %u\\n", opcode);');
    elif 'MI->getOpcode()' in line:
        if 'switch' in line:
            line2 = line.replace('MI->getOpcode()', 'MCInst_getOpcode(MI)')
        else:
            line2 = line.replace('MI->getOpcode()', 'opcode')
        print_line(line2)

    elif 'O << ' in line:
        if '"' in line:
            line2 = line.lower()
            line2 = line2.replace('o << ', 'SStream_concat0(O, ');
        else:
            line2 = line.replace('O << ', 'SStream_concat0(O, ');
        line2 = line2.replace("'", '"')
        line2 = line2.replace(';', ');')
        if '" : "' in line2:    # "segment : offset" in X86
            line2 = line2.replace('" : "', '":"')

        # ARM
        print_line(line2)

        if '", #0"' in line2:
            print_line('    op_addImm(MI, 0);')

        if '", #1"' in line2:
            print_line('    op_addImm(MI, 1);')

        # PowerPC
        if '", 268"' in line2:
            print_line('    op_addImm(MI, 268);')

        elif '", 256"' in line2:
            print_line('    op_addImm(MI, 256);')

        elif '", 0, "' in line2 or '", 0"' in line2:
            print_line('    op_addImm(MI, 0);')

        elif '", -1"' in line2:
            print_line('    op_addImm(MI, -1);')
        

        if '], [' in line2 or ']!, [' in line2:
            print_line('    set_mem_access(MI, false);')
            print_line('    set_mem_access(MI, true);')
        
        elif "\"[\"" in line2:
            # Check for SME_Index specific string of only "["
            print_line('    set_sme_index(MI, true);')

        elif '[' in line2:
            if not '[]' in line2:
                print_line('    set_mem_access(MI, true);')

        elif ']' in line2:
            if not '[]' in line2:
                print_line('    set_mem_access(MI, false);')

        if '".f64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F64);')
        elif '".f32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F32);')
        elif '".f16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F16);')
        elif '".s64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S64);')
        elif '".s32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S32);')
        elif '".s16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S16);')
        elif '".s8\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S8);')
        elif '".u64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U64);')
        elif '".u32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U32);')
        elif '".u16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U16);')
        elif '".u8\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U8);')
        elif '".i64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_I64);')
        elif '".i32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_I32);')
        elif '".i16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_I16);')
        elif '".i8\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_I8);')
        elif '".f16.f64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F16F64);')
        elif '".f64.f16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F64F16);')
        elif '".f16.f32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F16F32);')
        elif '".f32.f16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F32F16);')
        elif '".f64.f32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F64F32);')
        elif '".f32.f64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F32F64);')
        elif '".s32.f32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S32F32);')
        elif '".f32.s32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F32S32);')
        elif '".u32.f32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U32F32);')
        elif '".f32.u32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F32U32);')
        elif '".p8\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_P8);')
        elif '".f64.s16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F64S16);')
        elif '".s16.f64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S16F64);')
        elif '".f32.s16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F32S16);')
        elif '".s16.f32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S16F32);')
        elif '".f64.s32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F64S32);')
        elif '".s32.f64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_S32F64);')
        elif '".f64.u16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F64U16);')
        elif '".u16.f64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U16F64);')
        elif '".f32.u16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F32U16);')
        elif '".u16.f32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U16F32);')
        elif '".f64.u32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F64U32);')
        elif '".u32.f64\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U32F64);')
        elif '".f16.u32\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F16U32);')
        elif '".u32.f16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U32F16);')
        elif '".f16.u16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_F16U16);')
        elif '".u16.f16\\t"' in line2:
            print_line('    ARM_addVectorDataType(MI, ARM_VECTORDATA_U16F16);')
        elif '"\\tlr"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_LR);')
        elif '"\\tapsr_nzcv, fpscr"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_APSR_NZCV);')
            print_line('    ARM_addReg(MI, ARM_REG_FPSCR);')
        elif '"\\tpc, lr"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_PC);')
            print_line('    ARM_addReg(MI, ARM_REG_LR);')
        elif '"\\tfpscr, "' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPSCR);')
        elif '"\\tfpexc, "' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPEXC);')
        elif '"\\tfpinst, "' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPINST);')
        elif '"\\tfpinst2, "' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPINST2);')
        elif '"\\tfpsid, "' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPSID);')
        elif '"\\tsp, "' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_SP);')
        elif '"\\tsp!, "' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_SP);')
        elif '", apsr"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_APSR);')
        elif '", spsr"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_SPSR);')
        elif '", fpscr"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPSCR);')
        elif '", fpscr"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPSCR);')
        elif '", fpexc"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPEXC);')
        elif '", fpinst"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPINST);')
        elif '", fpinst2"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPINST2);')
        elif '", fpsid"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_FPSID);')
        elif '", mvfr0"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_MVFR0);')
        elif '", mvfr1"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_MVFR1);')
        elif '", mvfr2"' in line2:
            print_line('    ARM_addReg(MI, ARM_REG_MVFR2);')
        elif '.8\\t' in line2:
            print_line('    ARM_addVectorDataSize(MI, 8);')
        elif '.16\\t' in line2:
            print_line('    ARM_addVectorDataSize(MI, 16);')
        elif '.32\\t' in line2:
            print_line('    ARM_addVectorDataSize(MI, 32);')
        elif '.64\\t' in line2:
            print_line('    ARM_addVectorDataSize(MI, 64);')
        elif '" ^"' in line2:
            print_line('    ARM_addUserMode(MI);')

        if '.16b' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_16B);')
        elif '.8b' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_8B);')
        elif '.4b' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_4B);')
        elif '.b' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_1B);')
        elif '.8h' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_8H);')
        elif '.4h' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_4H);')
        elif '.2h' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_2H);')
        elif '.h' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_1H);')
        elif '.4s' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_4S);')
        elif '.2s' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_2S);')
        elif '.s' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_1S);')
        elif '.2d' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_2D);')
        elif '.1d' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_1D);')
        elif '.1q' in line2:
            print_line('    arm64_op_addVectorArrSpecifier(MI, ARM64_VAS_1Q);')

        if '#0.0' in line2:
            print_line('    arm64_op_addFP(MI, 0);')
        elif '#0' in line2:
            print_line('    arm64_op_addImm(MI, 0);')
        elif '#8' in line2:
            print_line('    arm64_op_addImm(MI, 8);')
        elif '#16' in line2:
            print_line('    arm64_op_addImm(MI, 16);')
        elif '#32' in line2:
            print_line('    arm64_op_addImm(MI, 32);')

        # X86
        if '", %rax"' in line2 or '", rax"' in line2:
            print_line('    op_addReg(MI, X86_REG_RAX);')
        elif '", %eax"' in line2 or '", eax"' in line2:
            print_line('    op_addReg(MI, X86_REG_EAX);')
        elif '", %ax"' in line2 or '", ax"' in line2:
            print_line('    op_addReg(MI, X86_REG_AX);')
        elif '", %al"' in line2 or '", al"' in line2:
            print_line('    op_addReg(MI, X86_REG_AL);')
        elif '", %dx"' in line2 or '", dx"' in line2:
            print_line('    op_addReg(MI, X86_REG_DX);')
        elif '", %st(0)"' in line2 or '", st(0)"' in line2:
            print_line('    op_addReg(MI, X86_REG_ST0);')
        elif '", 1"' in line2:
            print_line('    op_addImm(MI, 1);')
        elif '", cl"' in line2:
            print_line('    op_addReg(MI, X86_REG_CL);')
        elif '"{1to2}, "' in line2:
            print_line('    op_addAvxBroadcast(MI, X86_AVX_BCAST_2);')
        elif '"{1to4}, "' in line2:
            print_line('    op_addAvxBroadcast(MI, X86_AVX_BCAST_4);')
        elif '"{1to8}, "' in line2:
            print_line('    op_addAvxBroadcast(MI, X86_AVX_BCAST_8);')
        elif '"{1to16}, "' in line2:
            print_line('    op_addAvxBroadcast(MI, X86_AVX_BCAST_16);')
        elif '{z}{sae}' in line2:
            print_line('    op_addAvxSae(MI);')
            print_line('    op_addAvxZeroOpmask(MI);')
        elif ('{z}' in line2):
            print_line('    op_addAvxZeroOpmask(MI);')
        elif '{sae}' in line2:
            print_line('    op_addAvxSae(MI);')
    elif 'llvm_unreachable("Invalid command number.");' in line:
        line2 = line.replace('llvm_unreachable("Invalid command number.");', '// unreachable')
        print_line(line2)
    elif ('assert(' in line) or ('assert (' in line):
        pass
    elif 'Invalid alt name index' in line:
        pass
    elif '::' in line and 'case ' in line:
        #print_line(line2)
        print_line(line)
    elif 'MI->getNumOperands()' in line:
        line2 = line.replace('MI->getNumOperands()', 'MCInst_getNumOperands(MI)')
        print_line(line2)
    elif 'const MCOperand &MCOp' in line:
        line2 = line.replace('const MCOperand &MCOp', 'MCOperand *MCOp')
        print_line(line2)
    elif 'MI->getOperand(0).isImm()' in line:
        line2 = line.replace('MI->getOperand(0).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 0))')
        print_line(line2)
    elif 'MI->getOperand(1).isImm()' in line:
        line2 = line.replace('MI->getOperand(1).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 1))')
        print_line(line2)
    elif 'MI->getOperand(2).isImm()' in line:
        line2 = line.replace('MI->getOperand(2).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 2))')
        print_line(line2)
    elif 'MI->getOperand(3).isImm()' in line:
        line2 = line.replace('MI->getOperand(3).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 3))')
        print_line(line2)
    elif 'MI->getOperand(4).isImm()' in line:
        line2 = line.replace('MI->getOperand(4).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 4))')
        print_line(line2)
    elif 'MI->getOperand(5).isImm()' in line:
        line2 = line.replace('MI->getOperand(5).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 5))')
        print_line(line2)
    elif 'MI->getOperand(6).isImm()' in line:
        line2 = line.replace('MI->getOperand(6).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 6))')
        print_line(line2)
    elif 'MI->getOperand(7).isImm()' in line:
        line2 = line.replace('MI->getOperand(7).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 7))')
        print_line(line2)
    elif 'MI->getOperand(8).isImm()' in line:
        line2 = line.replace('MI->getOperand(8).isImm()', 'MCOperand_isImm(MCInst_getOperand(MI, 8))')
        print_line(line2)
    elif 'MI->getOperand(0).getImm()' in line:
        line2 = line.replace('MI->getOperand(0).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 0))')
        print_line(line2)
    elif 'MI->getOperand(1).getImm()' in line:
        line2 = line.replace('MI->getOperand(1).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 1))')
        print_line(line2)
    elif 'MI->getOperand(2).getImm()' in line:
        line2 = line.replace('MI->getOperand(2).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 2))')
        print_line(line2)
    elif 'MI->getOperand(3).getImm()' in line:
        line2 = line.replace('MI->getOperand(3).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 3))')
        print_line(line2)
    elif 'MI->getOperand(4).getImm()' in line:
        line2 = line.replace('MI->getOperand(4).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 4))')
        print_line(line2)
    elif 'MI->getOperand(5).getImm()' in line:
        line2 = line.replace('MI->getOperand(5).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 5))')
        print_line(line2)
    elif 'MI->getOperand(6).getImm()' in line:
        line2 = line.replace('MI->getOperand(6).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 6))')
        print_line(line2)
    elif 'MI->getOperand(7).getImm()' in line:
        line2 = line.replace('MI->getOperand(7).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 7))')
        print_line(line2)
    elif 'MI->getOperand(8).getImm()' in line:
        line2 = line.replace('MI->getOperand(8).getImm()', 'MCOperand_getImm(MCInst_getOperand(MI, 8))')
        print_line(line2)
    elif 'MRI.getRegClass(' in line:
        classid = extract_paren(line, 'getRegClass(')
        operand = extract_paren(line, 'getOperand')
        line2 = line.replace('MI->getNumOperands()', 'MCInst_getNumOperands(MI)')
        line2 = '        GETREGCLASS_CONTAIN(%s, %s)' %(classid, operand)
        if line.endswith('())) {'):
            line2 += ') {'
        elif line.endswith(' {'):
            line2 += ' {'
        elif line.endswith(' &&'):
            line2 += ' &&'
        print_line(line2)
    elif 'MI->getOperand(' in line and 'isReg' in line:
        operand = extract_paren(line, 'getOperand')
        line2 = '        MCOperand_isReg(MCInst_getOperand(MI, %s))' %(operand)
        # MI->getOperand(1).isReg() &&
        if line.endswith(' {'):
            line2 += ' {'
        elif line.endswith(' &&'):
            line2 += ' &&'
        print_line(line2)
    elif 'MI->getOperand(' in line and 'getReg' in line:
        line2 = replace_getReg(line)
        # one more time
        line2 = replace_getReg(line2)
        print_line(line2)
    elif '    return false;' in line and in_printAliasInstr:
        print_line('    return NULL;')
    elif 'MCOp.isImm()' in line:
        line2 = line.replace('MCOp.isImm()', 'MCOperand_isImm(MCOp)')
        print_line(line2)
    elif 'MCOp.getImm()' in line:
        line2 = line.replace('MCOp.getImm()', 'MCOperand_getImm(MCOp)')
        if 'int64_t Val =' in line:
            line2 = line2.replace('int64_t Val =', 'Val =')
        print_line(line2)
    elif 'isSVEMaskOfIdenticalElements<' in line:
        if 'int8' in line:
            line2 = line.replace('isSVEMaskOfIdenticalElements', 'isSVEMaskOfIdenticalElements8')
            line2 = line2.replace('<int8_t>', '')
        elif 'int16' in line:
            line2 = line.replace('isSVEMaskOfIdenticalElements', 'isSVEMaskOfIdenticalElements16')
            line2 = line2.replace('<int16_t>', '')
        elif 'int32' in line: 
            line2 = line.replace('isSVEMaskOfIdenticalElements', 'isSVEMaskOfIdenticalElements32')
            line2 = line2.replace('<int32_t>', '')
        else:
            line2 = line.replace('isSVEMaskOfIdenticalElements', 'isSVEMaskOfIdenticalElements64')
            line2 = line2.replace('<int64_t>', '')
        print_line(line2)
    elif 'switch (PredicateIndex) {' in line:
        print_line('  int64_t Val;')
        print_line(line)
    elif 'uint32_t(' in line and in_printAliasInstr:
        line = line.replace('uint32_t(', '')
        line = line.replace(')', '')
        print_line(line)
    elif '#ifndef NDEBUG' in line and in_printAliasInstr:
        print_line("""
  char *AsmString;
  const size_t OpToSize = sizeof(OpToPatterns) / sizeof(PatternsForOpcode);

  const unsigned opcode = MCInst_getOpcode(MI);

  // Check for alias
  int OpToIndex = 0;
  for(int i = 0; i < OpToSize; i++){
    if(OpToPatterns[i].Opcode == opcode){
      OpToIndex = i;
      break;
    }
  }
  // Chech for match
  if(opcode != OpToPatterns[OpToIndex].Opcode)
    return NULL;

  const PatternsForOpcode opToPat = OpToPatterns[OpToIndex];

  // Try all patterns for this opcode
  uint32_t AsmStrOffset = ~0U;
  int patIdx = opToPat.PatternStart;
  while(patIdx < (opToPat.PatternStart + opToPat.NumPatterns)){
    // Check operand count first
    if(MCInst_getNumOperands(MI) != Patterns[patIdx].NumOperands)
      return NULL;
    
    // Test all conditions for this pattern
    int condIdx = Patterns[patIdx].AliasCondStart;
    int opIdx = 0;
    bool allPass = true;
    while(condIdx < (Patterns[patIdx].AliasCondStart + Patterns[patIdx].NumConds)){
      MCOperand *opnd = MCInst_getOperand(MI, opIdx);
      opIdx++;
      // Not concerned with any Feature related conditions as STI is disregarded
      switch (Conds[condIdx].Kind)
      {
      case AliasPatternCond_K_Ignore :
        // Operand can be anything.
        break;
      case AliasPatternCond_K_Reg :
        // Operand must be a specific register.
        allPass = allPass && (MCOperand_isReg(opnd) && MCOperand_getReg(opnd) == Conds[condIdx].Value);
        break;
      case AliasPatternCond_K_TiedReg :
        // Operand must match the register of another operand.
        allPass = allPass && (MCOperand_isReg(opnd) && MCOperand_getReg(opnd) == 
                  MCOperand_getReg(MCInst_getOperand(MI, Conds[condIdx].Value)));
        break;
      case AliasPatternCond_K_Imm :
        // Operand must be a specific immediate.
        allPass = allPass && (MCOperand_isImm(opnd) && MCOperand_getImm(opnd) == Conds[condIdx].Value);
        break;
      case AliasPatternCond_K_RegClass :
        // Operand must be a register in this class. Value is a register class id.
        allPass = allPass && (MCOperand_isReg(opnd) && GETREGCLASS_CONTAIN(Conds[condIdx].Value, (opIdx-1)));
        break;
      case AliasPatternCond_K_Custom :
        // Operand must match some custom criteria.
        allPass = allPass && AArch64InstPrinterValidateMCOperand(opnd, Conds[condIdx].Value);
        break;
      case AliasPatternCond_K_Feature :
      case AliasPatternCond_K_NegFeature :
      case AliasPatternCond_K_OrFeature :
      case AliasPatternCond_K_OrNegFeature :
      case AliasPatternCond_K_EndOrFeatures :
      default :
        break;
      }
      condIdx++;
    }
    if(allPass){
      AsmStrOffset = Patterns[patIdx].AsmStrOffset;
      break;
    }
    patIdx++;
  }

  // If no alias matched, don't print an alias.
  if (AsmStrOffset == ~0U)
    return NULL;

  AsmString = cs_strdup(&AsmStrings[AsmStrOffset]);

  tmpString = cs_strdup(AsmString);

  while (AsmString[I] != ' ' && AsmString[I] != '\\t' &&
        AsmString[I] != '$' && AsmString[I] != '\\0')
    ++I;

  tmpString[I] = 0;
  SStream_concat0(OS, tmpString);

  if (AsmString[I] != '\\0') {
    if (AsmString[I] == ' ' || AsmString[I] == '\\t') {
      SStream_concat0(OS, " ");
      ++I;
    }

    bool isSME = false;
    do {
      if (AsmString[I] == '$') {
        ++I;
        if (AsmString[I] == (char)0xff) {
          ++I;
          OpIdx = AsmString[I++] - 1;
          PrintMethodIdx = AsmString[I++] - 1;
          printCustomAliasOperand(MI, 0, OpIdx, PrintMethodIdx, OS);
        } else
          printOperand(MI, (unsigned)(AsmString[I++]) - 1, OS);
      } else {
        if (AsmString[I] == '[') {
          if (AsmString[I-1] != ' ') {
            set_sme_index(MI, true);
            isSME = true;
          } else {
            set_mem_access(MI, true);
          }
        } else if (AsmString[I] == ']') {
          if (isSME) {
            set_sme_index(MI, false);
            isSME = false;
          } else {
            set_mem_access(MI, false);
          }
        }
        SStream_concat1(OS, AsmString[I++]);
      }
    } while (AsmString[I] != '\\0');
  }
  cs_mem_free(AsmString);
  return tmpString;
}
        """)
        in_printAliasInstr = False
        # skip next few lines
        skip_printing = True
    elif '::printCustomAliasOperand' in line:
        # print again
        skip_printing = False
        print_line('static void printCustomAliasOperand(')
    elif 'const MCSubtargetInfo &STI' in line:
        pass
    elif 'const MCInst *MI' in line:
        line2 = line.replace('const MCInst *MI', 'MCInst *MI')
        print_line(line2)
    elif 'llvm_unreachable("' in line:
        if 'default: ' in line:
            print_line('  default:')
        elif 'llvm_unreachable("Unknown MCOperandPredicate kind")' in line:
            print_line('    return false; // never reach')
        else:
            pass
    elif 'raw_ostream &' in line:
        line2 = line.replace('raw_ostream &', 'SStream *')
        if line2.endswith(' {'):
            line2 = line2.replace(' {', '\n{')
        print_line(line2)
    elif 'printPredicateOperand(' in line and 'STI, ' in line:
        line2 = line.replace('STI, ', '')
        print_line(line2)
    elif '// Fragment ' in line:
        # // Fragment 0 encoded into 6 bits for 51 unique commands.
        tmp = line.strip().split(' ')
        fragment_no = tmp[2]
        print_line(line)
    elif ('switch ((' in line or 'if ((' in line) and 'Bits' in line:
        # switch ((Bits >> 14) & 63) {
        bits = line.strip()
        bits = bits.replace('switch ', '')
        bits = bits.replace('if ', '')
        bits = bits.replace('{', '')
        bits = bits.strip()
        print_line('  // printf("Fragment %s: %%"PRIu64"\\n", %s);' %(fragment_no, bits))
        print_line(line)
    elif not skip_printing:
        print_line(line)

    if line == '  };':
        if need_endif and not in_getRegisterName:
            # endif only for AsmStrs when we are not inside getRegisterName()
            print_line("#endif")
            need_endif = False
    elif 'return AsmStrs+RegAsmOffset[RegNo-1];' in line:
        if in_getRegisterName:
            # return NULL for register name on Diet mode
            print_line("#else")
            print_line("  return NULL;")
            print_line("#endif")
            print_line("}")
            need_endif = False
            in_getRegisterName = False
            # skip 1 line
            skip_line = 1
    elif line == '  }':
        # ARM64
        if in_getRegisterName:
            # return NULL for register name on Diet mode
            print_line("#else")
            print_line("  return NULL;")
            print_line("#endif")
            print_line("}")
            need_endif = False
            in_getRegisterName = False
            # skip 1 line
            skip_line = 1
    elif 'default:' in line:
        # ARM64
        if in_getRegisterName:
            # get the size of RegAsmOffsetvreg[]
            print_line("    return (const char *)(sizeof(RegAsmOffsetvreg)/sizeof(RegAsmOffsetvreg[0]));")


f1.close()
f2.close()



#!/usr/bin/python
# By Nguyen Anh Quynh, 2015
# This tool extract sizes of immediadte operands from X86 instruction names.
# Syntax: ./autogen_x86imm.py

# Gather immediate sizes to put into X86ImmSize.inc
OUTPUT = "../arch/X86/X86ImmSize.inc"

f = open("../arch/X86/X86GenInstrInfo.inc")
f2 = open(OUTPUT, "w")
for line in f.readlines():
   tmp = line.strip().split("=")
   if len(tmp) == 2:    # X86_xxx = nnn,
       name = tmp[0].strip()
       if name == "X86_INSTRUCTION_LIST_END":   # no more instructions
           break
       if name.endswith("_DB"): # pseudo instruction
           continue
       if "_LOCK_" in name or "BEXTR" in name:  # exception
           continue
       if name.startswith("X86_"):  # instruction
           if name.endswith("16mi8"):
               f2.write("{2, %s},\n" %name)
           elif name.endswith("16ri8"):
               f2.write("{2, %s},\n" %name)
           elif name.endswith("32ri8"):
               f2.write("{4, %s},\n" %name)
           elif name.endswith("32mi8"):
               f2.write("{4, %s},\n" %name)
           elif name.endswith("64i32"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64mi32"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64ri32"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64ri8"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64mi8"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("16rmi8"):
               f2.write("{2, %s},\n" %name)
           elif name.endswith("32rmi8"):
               f2.write("{4, %s},\n" %name)
           elif name.endswith("16rri8"):
               f2.write("{2, %s},\n" %name)
           elif name.endswith("32rri8"):
               f2.write("{4, %s},\n" %name)
           elif name.endswith("64rmi8"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64rmi32"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64rri32"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64rri8"):
               f2.write("{8, %s},\n" %name)
           elif name.endswith("32ri64"):    # special case
               f2.write("{8, %s},\n" %name)
           elif name.endswith("16i8"):    # special case
               f2.write("{2, %s},\n" %name)
           elif name.endswith("32i8"):    # special case
               f2.write("{4, %s},\n" %name)
           elif name.endswith("64i16"):    # special case
               f2.write("{8, %s},\n" %name)
           elif name.endswith("64i8"):    # special case
               f2.write("{8, %s},\n" %name)

           elif name.endswith("i8") or "i8_" in name:
               f2.write("{1, %s},\n" %name)
           elif "8ri" in name or "8mi" in name:
               f2.write("{1, %s},\n" %name)

           elif name.endswith("i16") or "i16_" in name:
               f2.write("{2, %s},\n" %name)
           elif "16ri" in name or "16mi" in name:
               f2.write("{2, %s},\n" %name)

           elif name.endswith("i32") or "i32_" in name:
               f2.write("{4, %s},\n" %name)
           elif "32ri" in name or "32mi" in name:
               f2.write("{4, %s},\n" %name)

           elif name.endswith("i64") or "i64_" in name:
               f2.write("{8, %s},\n" %name)
           elif "64ri" in name or "64mi" in name:
               f2.write("{8, %s},\n" %name)

f.close()
f2.close()

print("Generated %s" %OUTPUT)



#!/usr/bin/python
# print list of instructions LLVM inc files, for Capstone disassembler.
# this will be put into capstone/<arch>.h
# by Nguyen Anh Quynh, 2019

import sys

if len(sys.argv) == 1:
    print("Syntax: %s <GenAsmMatcher.inc> <GenInstrInfo.inc> MappingInsn.inc" %sys.argv[0])
    sys.exit(1)

# MappingInsn.inc
f = open(sys.argv[3])
mapping = f.readlines()
f.close()

print("""/* Capstone Disassembly Engine, http://www.capstone-engine.org */
/* This is auto-gen data for Capstone disassembly engine (www.capstone-engine.org) */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2013-2019 */
""")

# lib/Target/X86/X86GenAsmMatcher.inc
# static const MatchEntry MatchTable1[] = {
#  { 0 /* aaa */, X86::AAA, Convert_NoOperands, Feature_Not64BitMode, {  }, },

# extract insn from GenAsmMatcher Table
# return (arch, mnem, insn_id)
def extract_insn(line):
    tmp = line.split(',')
    insn_raw = tmp[1].strip()
    insn_mnem = tmp[0].split(' ')[3]
    # X86 mov.s
    if '.' in insn_mnem:
        tmp = insn_mnem.split('.')
        insn_mnem = tmp[0]
    tmp = insn_raw.split('::')
    arch = tmp[0]
    # AArch64 -> ARM64
    if arch.upper() == 'AArch64':
        arch = 'ARM64'
    return (arch, insn_mnem, tmp[1])



# extract all insn lines from GenAsmMatcher
# return arch, insn_id_list, insn_lines
def extract_matcher(filename):
    f = open(filename)
    lines = f.readlines()
    f.close()

    match_count = 0
    #insn_lines = []
    insn_id_list = {}
    arch = None
    first_insn = None

    pattern = None
    # first we try to find Table1, or Table0
    for line in lines:
        if 'MatchEntry MatchTable0[] = {' in line.strip():
            pattern = 'MatchEntry MatchTable0[] = {'
        elif 'MatchEntry MatchTable1[] = {' in line.strip():
            pattern = 'MatchEntry MatchTable1[] = {'
            # last pattern, done
            break

    # 1st enum is register enum
    for line in lines:
        line = line.rstrip()
    
        if len(line.strip()) == 0:
            continue
    
        if pattern in line.strip():
            match_count += 1
            #print(line.strip())
            continue
    
        line = line.strip()
        if match_count == 1:
            if line == '};':
                # done with first enum
                break
            else:
                _arch, mnem, insn_id = extract_insn(line)
                if not mnem.startswith('__'):
                    if not first_insn:
                        arch, first_insn = _arch, insn_id
                    if not insn_id in insn_id_list:
                        # print("***", arch, mnem, insn_id)
                        insn_id_list[insn_id] = mnem
                        #insn_lines.append(line)

    #return arch, first_insn, insn_id_list, insn_lines
    return arch, first_insn, insn_id_list

# GenAsmMatcher.inc
#arch, first_insn, insn_id_list, match_lines = extract_matcher(sys.argv[1])
arch, first_insn, insn_id_list = extract_matcher(sys.argv[1])
arch = arch.upper()

#for line in insn_id_list:
#    print(line)


insn_list = []
#{
#        X86_AAA, X86_INS_AAA,
##ifndef CAPSTONE_DIET
#        { 0 }, { 0 }, { X86_GRP_NOT64BITMODE, 0 }, 0, 0
##endif
#},
def print_entry(arch, insn_id, mnem, mapping, mnem_can_be_wrong):
    print(arch, insn_id, mnem, mnem_can_be_wrong)
    if not mnem_can_be_wrong:
        insn = "%s_INS_%s" %(arch.upper(), mnem.upper())
        if insn in insn_list:
            return
        print("%s," %insn)
        insn_list.append(insn)
        return

    insn = "%s_%s" %(arch.upper(), insn_id)
    # so mnem can be wrong, we need to verify with MappingInsn.inc
    # first, try to find this entry in old MappingInsn.inc file
    for i in range(len(mapping)):
        tmp = mapping[i].split(',')
        if tmp[0].strip() == insn:
            insn = tmp[1].strip()
            if insn in insn_list:
                return
            #print("==== get below from MappingInsn.inc file: %s" %insn)
            print("%s," %insn)
            insn_list.append(insn)
            return


# extract from GenInstrInfo.inc, because the insn id is in order
enum_count = 0
meet_insn = False

# GenInstrInfo.inc
f = open(sys.argv[2])
lines = f.readlines()
f.close()

count = 0
last_mnem = None

# 1st enum is register enum
for line in lines:
    line = line.rstrip()

    if len(line.strip()) == 0:
        continue

    if line.strip() == 'enum {':
        enum_count += 1
        #print(line.strip())
        continue

    line = line.strip()
    if enum_count == 1:
        if 'INSTRUCTION_LIST_END' in line:
            break
        else:
            insn = None
            if meet_insn:
                # enum items
                insn = line.split('=')[0].strip()
                if 'CALLSTACK' in insn or 'TAILJUMP' in insn:
                    # pseudo instruction
                    insn = None

            elif line.startswith(first_insn):
                insn = line.split('=')[0].strip()
                meet_insn = True

            if insn:
                count += 1
                if insn == 'BSWAP16r_BAD':
                    last_mnem = 'BSWAP'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'CMOVNP_Fp32':
                    last_mnem = 'FCMOVNP'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'CMOVP_Fp3':
                    last_mnem = 'FCMOVP'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'CMPSDrm_Int':
                    last_mnem = 'CMPSD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'MOVSX16rm16':
                    last_mnem = 'MOVSX'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'MOVZX16rm16':
                    last_mnem = 'MOVZX'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'ST_Fp32m':
                    last_mnem = 'FST'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'CMOVNP_Fp64':
                    last_mnem = 'FCMOVNU'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'CMPSDrr_Int':
                    last_mnem = 'CMPSD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'CMPSSrm_Int':
                    last_mnem = 'CMPSS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VCMPSDrm_Int':
                    last_mnem = 'VCMPSD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VCMPSSrm_Int':
                    last_mnem = 'VCMPSS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VPCMOVYrrr_REV':
                    last_mnem = 'VPCMOV'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VRNDSCALESDZm':
                    last_mnem = 'VRNDSCALESD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VRNDSCALESSZm':
                    last_mnem = 'VRNDSCALESS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMAXCPDZ128rm':
                    last_mnem = 'VMAXPD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMAXCPSZ128rm':
                    last_mnem = 'VMAXPS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMAXCSDZrm':
                    last_mnem = 'VMAXSD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMAXCSSZrm':
                    last_mnem = 'VMAXSS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMINCPDZ128rm':
                    last_mnem = 'VMINPD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMINCPSZ128rm':
                    last_mnem = 'VMINPS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMINCSDZrm':
                    last_mnem = 'VMINSD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMINCSSZrm':
                    last_mnem = 'VMINSS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VMOV64toPQIZrm':
                    last_mnem = 'VMOVQ'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VPERMIL2PDYrr_REV':
                    last_mnem = 'VPERMILPD'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VPERMIL2PSYrr_REV':
                    last_mnem = 'VPERMILPS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VCVTSD2SI64Zrm_Int':
                    last_mnem = 'VCVTSD2SI'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn == 'VCVTSD2SSrm_Int':
                    last_mnem = 'VCVTSD2SS'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn ==    'VCVTSS2SI64Zrm_Int':
                    last_mnem = 'VCVTSS2SI'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn ==    'VCVTTSD2SI64Zrm_Int':
                    last_mnem = 'VCVTTSD2SI'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                elif insn ==    'VCVTTSS2SI64Zrm_Int':
                    last_mnem = 'VCVTTSS2SI'
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)

                elif insn.startswith('VFMSUBADD'):
                    if insn[len('VFMSUBADD')].isdigit():
                        last_mnem = insn[:len('VFMSUBADD123xy')]
                    else:
                        last_mnem = insn[:len('VFMSUBADDSS')]
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)

                elif insn.startswith('VFMADDSUB'):
                    if insn[len('VFMADDSUB')].isdigit():
                        last_mnem = insn[:len('VFMADDSUB123xy')]
                    else:
                        last_mnem = insn[:len('VFMADDSUBSS')]
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)

                elif insn.startswith('VFMADD'):
                    if insn[len('VFMADD')].isdigit():
                        last_mnem = insn[:len('VFMADD123PD')]
                    else:
                        last_mnem = insn[:len('VFMADDPD')]
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)

                elif insn.startswith('VFMSUB'):
                    if insn[len('VFMSUB')].isdigit():
                        last_mnem = insn[:len('VFMSUB123PD')]
                    else:
                        last_mnem = insn[:len('VFMSUBPD')]
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)

                elif insn.startswith('VFNMADD'):
                    if insn[len('VFNMADD')].isdigit():
                        last_mnem = insn[:len('VFNMADD123xy')]
                    else:
                        last_mnem = insn[:len('VFNMADDSS')]
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)

                elif insn.startswith('VFNMSUB'):
                    if insn[len('VFNMSUB')].isdigit():
                        last_mnem = insn[:len('VFNMSUB123xy')]
                    else:
                        last_mnem = insn[:len('VFNMSUBSS')]
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)

                elif insn in insn_id_list:
                    # trust old mapping table
                    last_mnem = insn_id_list[insn].upper()
                    print_entry(arch.upper(), insn, last_mnem, mapping, False)
                else:
                    # the last option when we cannot find mnem: use the last good mnem
                    print_entry(arch.upper(), insn, last_mnem, mapping, True)
